from typing import Any
from typing import List
from typing import Optional
from typing import Text
from typing import Union

import ruamel.yaml
from ruamel.yaml.representer import RoundTripRepresenter

from ruamel_yaml_line_info.constructor import BinaryInt
from ruamel_yaml_line_info.constructor import DoubleQuotedScalarString
from ruamel_yaml_line_info.constructor import FoldedScalarString
from ruamel_yaml_line_info.constructor import HexCapsInt
from ruamel_yaml_line_info.constructor import HexInt
from ruamel_yaml_line_info.constructor import LiteralScalarString
from ruamel_yaml_line_info.constructor import OctalInt
from ruamel_yaml_line_info.constructor import PlainScalarString
from ruamel_yaml_line_info.constructor import RoundTripConstructor
from ruamel_yaml_line_info.constructor import ScalarBoolean
from ruamel_yaml_line_info.constructor import ScalarFloat
from ruamel_yaml_line_info.constructor import ScalarInt
from ruamel_yaml_line_info.constructor import SingleQuotedScalarString


class YAML(ruamel.yaml.YAML):
    """Subclass of ruamel.yaml.YAML that keeps line numbers."""

    def __init__(
        self: Any,
        *,
        typ: Optional[Union[List[Text], Text]] = None,
        pure: Any = False,
        output: Any = None,
        plug_ins: Any = None,
    ) -> None:  # input=None,
        """Alternative constructor to ruamel.yaml.YAML that keeps line numbers."""
        super().__init__(typ=typ, pure=pure, output=output, plug_ins=plug_ins)
        YAML.with_line_numbers(yaml=self)

    @classmethod
    def with_line_numbers(cls, yaml: ruamel.yaml.YAML) -> ruamel.yaml.YAML:
        """Updates a ruamel.yaml.YAML instance to keep line numbers."""
        yaml.Constructor = RoundTripConstructor

        # re-add so that we override the parent class' default constructor
        yaml.Constructor.add_default_constructor("int")
        yaml.Constructor.add_default_constructor("float")
        yaml.Constructor.add_default_constructor("bool")

        # needed for representers
        RoundTripRepresenter.add_representer(
            LiteralScalarString,
            RoundTripRepresenter.represent_literal_scalarstring,
        )
        RoundTripRepresenter.add_representer(
            FoldedScalarString, RoundTripRepresenter.represent_folded_scalarstring
        )
        RoundTripRepresenter.add_representer(
            SingleQuotedScalarString,
            RoundTripRepresenter.represent_single_quoted_scalarstring,
        )
        RoundTripRepresenter.add_representer(
            DoubleQuotedScalarString,
            RoundTripRepresenter.represent_double_quoted_scalarstring,
        )
        RoundTripRepresenter.add_representer(
            PlainScalarString, RoundTripRepresenter.represent_plain_scalarstring
        )
        RoundTripRepresenter.add_representer(ScalarInt, RoundTripRepresenter.represent_scalar_int)
        RoundTripRepresenter.add_representer(BinaryInt, RoundTripRepresenter.represent_binary_int)
        RoundTripRepresenter.add_representer(OctalInt, RoundTripRepresenter.represent_octal_int)
        RoundTripRepresenter.add_representer(HexInt, RoundTripRepresenter.represent_hex_int)
        RoundTripRepresenter.add_representer(
            HexCapsInt, RoundTripRepresenter.represent_hex_caps_int
        )
        RoundTripRepresenter.add_representer(
            ScalarFloat, RoundTripRepresenter.represent_scalar_float
        )
        # RoundTripRepresenter.add_representer(
        #    ExponentialFloat, RoundTripRepresenter.represent_scalar_float
        # )
        # RoundTripRepresenter.add_representer(
        #    ExponentialCapsFloat, RoundTripRepresenter.represent_scalar_float
        # )
        RoundTripRepresenter.add_representer(
            ScalarBoolean, RoundTripRepresenter.represent_scalar_bool
        )
        return yaml



import math
import textwrap
from typing import Any
from typing import Optional

import pytest
import ruamel

import ruamel_yaml_line_info
from ruamel_yaml_line_info import YAML
from ruamel_yaml_line_info.constructor import DoubleQuotedScalarString
from ruamel_yaml_line_info.constructor import FoldedScalarString
from ruamel_yaml_line_info.constructor import HexCapsInt
from ruamel_yaml_line_info.constructor import HexInt
from ruamel_yaml_line_info.constructor import LiteralScalarString
from ruamel_yaml_line_info.constructor import OctalInt
from ruamel_yaml_line_info.constructor import PlainScalarString
from ruamel_yaml_line_info.constructor import ScalarFloat
from ruamel_yaml_line_info.constructor import ScalarInt
from ruamel_yaml_line_info.constructor import SingleQuotedScalarString


def _assert_line_col(node: Any, line: int, col: int, value: Any, typ: Optional[Any] = None) -> None:
    assert hasattr(node, "lc")
    assert node.lc.line == line
    assert node.lc.col == col
    if isinstance(value, float) and math.isnan(value):
        assert math.isnan(node)
    else:
        assert node == value
    if typ is not None:
        assert isinstance(node, typ), type(node)


def _round_trip_load(
    text: Any,
    preserve_quotes: Optional[bool] = None,
    version: Optional[Any] = None,
) -> Any:
    text = textwrap.dedent(text)
    yaml = YAML(typ="rt")
    yaml.preserve_quotes = preserve_quotes
    yaml.version = version
    return yaml.load(text)


def test_basic() -> None:
    """Tests basic parsing and line numbers."""
    text = """\
        foo: bar
        block: >
          this is not
          a multiline
          block
        single_quote: 'single'
        double_quote: "double"
        multi: |
          this really
          is a multiline
          block
        plain: vanilla
        first_int: 1
        second_int: 2
        ints:
          canonical: 685230
          decimal: +685_230
          octal: 0o2472256
          hexadecimal_upper: 0x_0A_74_AE
          hexadecimal_lower: 0x_0a_74_ae
          binary: 0b1010_0111_0100_1010_1110
        first_float: 1.2
        second_float: 4.3
        floats:
            canonical: 1.23015e+3
            exponential: 12.3015e+02
            fixed: 1230.15
            neginf: -.inf
            nan: .NaN
        bool_true: true
        bool_false: false
        """
    yaml = _round_trip_load(text, preserve_quotes=True)

    # strings
    _assert_line_col(yaml["foo"], line=0, col=5, value="bar", typ=PlainScalarString)
    _assert_line_col(
        yaml["block"],
        line=1,
        col=7,
        value="this is not a multiline block\n",
        typ=FoldedScalarString,
    )
    _assert_line_col(
        yaml["single_quote"], line=5, col=14, value="single", typ=SingleQuotedScalarString
    )
    _assert_line_col(
        yaml["double_quote"], line=6, col=14, value="double", typ=DoubleQuotedScalarString
    )
    _assert_line_col(
        yaml["multi"],
        line=7,
        col=7,
        value="this really\nis a multiline\nblock\n",
        typ=LiteralScalarString,
    )
    _assert_line_col(yaml["plain"], line=11, col=7, value="vanilla", typ=PlainScalarString)

    # integers
    _assert_line_col(yaml["first_int"], line=12, col=11, value=1, typ=ScalarInt)
    _assert_line_col(yaml["second_int"], line=13, col=12, value=2, typ=ScalarInt)
    _assert_line_col(yaml["ints"]["canonical"], line=15, col=13, value=685230, typ=ScalarInt)
    _assert_line_col(yaml["ints"]["decimal"], line=16, col=11, value=685230, typ=ScalarInt)
    _assert_line_col(yaml["ints"]["octal"], line=17, col=9, value=685230, typ=OctalInt)
    _assert_line_col(
        yaml["ints"]["hexadecimal_upper"], line=18, col=21, value=685230, typ=HexCapsInt
    )
    _assert_line_col(yaml["ints"]["hexadecimal_lower"], line=19, col=21, value=685230, typ=HexInt)
    _assert_line_col(yaml["ints"]["binary"], line=20, col=10, value=685230)

    # floats (these will all be floats or ScalarFloats)
    _assert_line_col(yaml["first_float"], line=21, col=13, value=1.2, typ=float)
    _assert_line_col(yaml["second_float"], line=22, col=14, value=4.3, typ=float)
    _assert_line_col(yaml["floats"]["canonical"], line=24, col=15, value=1.23015e3, typ=ScalarFloat)
    _assert_line_col(
        yaml["floats"]["exponential"], line=25, col=17, value=12.3015e02, typ=ScalarFloat
    )
    _assert_line_col(yaml["floats"]["fixed"], line=26, col=11, value=1230.15, typ=ScalarFloat)
    _assert_line_col(
        yaml["floats"]["neginf"], line=27, col=12, value=float("-inf"), typ=ScalarFloat
    )
    _assert_line_col(yaml["floats"]["nan"], line=28, col=9, value=float("nan"), typ=ScalarFloat)

    # booleans
    _assert_line_col(yaml["bool_true"], line=29, col=11, value=True)
    _assert_line_col(yaml["bool_false"], line=30, col=12, value=False)


# from PyYAML docs
class _Dice(tuple):
    def __new__(cls, a: int, b: int) -> "_Dice":
        return tuple.__new__(cls, [a, b])

    def __repr__(self) -> str:
        return "_Dice(%s,%s)" % self


def _dice_constructor(loader: Any, node: Any) -> _Dice:
    value = loader.construct_scalar(node)
    a, b = map(int, value.split("d"))
    return _Dice(a, b)


def test_dice_constructor() -> None:
    """Tests using a loader and custom constructor."""
    import ruamel.yaml  # NOQA

    with pytest.warns(PendingDeprecationWarning):
        yaml = ruamel.yaml.YAML(typ="unsafe", pure=True)
    ruamel.yaml.add_constructor("!dice", _dice_constructor)
    data = yaml.load("initial hit points: !dice 8d4")
    assert str(data) == "{'initial hit points': _Dice(8,4)}"


class _RoundTripLoader(
    ruamel.yaml.reader.Reader,
    ruamel.yaml.scanner.RoundTripScanner,
    ruamel.yaml.parser.RoundTripParser,
    ruamel.yaml.composer.Composer,
    ruamel_yaml_line_info.constructor.RoundTripConstructor,
    ruamel.yaml.resolver.VersionedResolver,
):
    def __init__(
        self,
        stream: Any,
        version: Optional[Any] = None,
        preserve_quotes: Optional[bool] = None,
    ) -> None:
        self.comment_handling = None  # issue 385
        ruamel.yaml.reader.Reader.__init__(self, stream, loader=self)
        ruamel.yaml.scanner.RoundTripScanner.__init__(self, loader=self)
        ruamel.yaml.parser.RoundTripParser.__init__(self, loader=self)
        ruamel.yaml.composer.Composer.__init__(self, loader=self)
        ruamel_yaml_line_info.RoundTripConstructor.__init__(
            self, preserve_quotes=preserve_quotes, loader=self
        )
        ruamel.yaml.resolver.VersionedResolver.__init__(self, version, loader=self)


def test_dice_constructor_with_loader() -> None:
    """Tests using a loader and custom constructor."""
    yaml = ruamel_yaml_line_info.YAML(typ="rt", pure=True)
    ruamel_yaml_line_info.constructor.RoundTripConstructor.add_constructor(
        "!dice",
        _dice_constructor,
    )
    data = yaml.load("initial hit points: !dice 8d4  # some comment")
    assert str(data) == "{'initial hit points': _Dice(8,4)}"
    value = data["initial hit points"]
    assert value == _Dice(8, 4)


def test_comment() -> None:
    """Tests using a loader and custom constructor."""
    yaml = ruamel_yaml_line_info.YAML(typ="rt", pure=True)
    data = yaml.load("foo:\n  # some comment\n  bar: cow\n")
    assert str(data) == "{'foo': {'bar': 'cow'}}"
    assert data["foo"].ca.comment[1][0].value == "# some comment\n"



from typing import Any
from typing import ClassVar
from typing import Optional
from typing import Union

import ruamel.yaml

__all__ = [
    "ScalarBoolean",
    "LiteralScalarString",
    "FoldedScalarString",
    "SingleQuotedScalarString",
    "DoubleQuotedScalarString",
    "PlainScalarString",
    # PreservedScalarString is the old name, as it was the first to be preserved on rt,
    # use LiteralScalarString instead
    "PreservedScalarString",
    "ScalarInt",
    "BinaryInt",
    "OctalInt",
    "HexInt",
    "HexCapsInt",
    "ScalarFloat",
    # Thes are not actually used by ruamel.yaml
    # "DecimalInt",
    # "ExponentialFloat",
    # "ExponentialCapsFloat",
]


class LiteralScalarString(ruamel.yaml.scalarstring.LiteralScalarString):
    """Sub-class of ruamel.yaml.scalarstring.LiteralScalarString to store line numbers."""

    __slots__ = ("comment", "lc")

    def __new__(cls, *args: Any, **kw: Any) -> Any:
        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
        ret_val = super().__new__(cls, *args, **kw)
        ret_val.lc = None
        return ret_val


class FoldedScalarString(ruamel.yaml.scalarstring.FoldedScalarString):
    """Sub-class of ruamel.yaml.scalarstring.FoldedScalarString to store line numbers."""

    __slots__ = ("fold_pos", "comment", "lc")

    def __new__(cls, *args: Any, **kw: Any) -> Any:
        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
        ret_val = super().__new__(cls, *args, **kw)
        ret_val.lc = None
        return ret_val


class DoubleQuotedScalarString(ruamel.yaml.scalarstring.DoubleQuotedScalarString):
    """Sub-class of ruamel.yaml.scalarstring.DoubleQuotedScalarString to store line numbers."""

    __slots__ = "lc"

    def __new__(cls, *args: Any, **kw: Any) -> Any:
        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
        ret_val = super().__new__(cls, *args, **kw)
        ret_val.lc = None
        return ret_val


class SingleQuotedScalarString(ruamel.yaml.scalarstring.SingleQuotedScalarString):
    """Sub-class of ruamel.yaml.scalarstring.SingleQuotedScalarString to store line numbers."""

    __slots__ = "lc"

    def __new__(cls, *args: Any, **kw: Any) -> Any:
        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
        ret_val = super().__new__(cls, *args, **kw)
        ret_val.lc = None
        return ret_val


class PlainScalarString(ruamel.yaml.scalarstring.PlainScalarString):
    """Sub-class of ruamel.yaml.scalarstring.PlainScalarString to store line numbers."""

    __slots__ = "lc"

    def __new__(cls, *args: Any, **kw: Any) -> Any:
        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
        ret_val = super().__new__(cls, *args, **kw)
        ret_val.lc = None
        return ret_val


PreservedScalarString = LiteralScalarString


class ScalarInt(ruamel.yaml.scalarint.ScalarInt):
    """Sub-class of ruamel.yaml.scalarint.ScalarInt to store line numbers."""

    def __new__(cls, *args: Any, **kw: Any) -> Any:
        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
        ret_val = super().__new__(cls, *args, **kw)
        ret_val.lc = None
        return ret_val


class BinaryInt(ruamel.yaml.scalarint.BinaryInt):
    """Sub-class of ruamel.yaml.scalarint.BinaryInt to store line numbers."""

    def __new__(cls, *args: Any, **kw: Any) -> Any:
        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
        ret_val = super().__new__(cls, *args, **kw)
        ret_val.lc = None
        return ret_val


class OctalInt(ruamel.yaml.scalarint.OctalInt):
    """Sub-class of ruamel.yaml.scalarint.OctalInt to store line numbers."""

    def __new__(cls, *args: Any, **kw: Any) -> Any:
        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
        ret_val = super().__new__(cls, *args, **kw)
        ret_val.lc = None
        return ret_val


class HexInt(ruamel.yaml.scalarint.HexInt):
    """Sub-class of ruamel.yaml.scalarint.HexInt to store line numbers."""

    def __new__(cls, *args: Any, **kw: Any) -> Any:
        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
        ret_val = super().__new__(cls, *args, **kw)
        ret_val.lc = None
        return ret_val


class HexCapsInt(ruamel.yaml.scalarint.HexCapsInt):
    """Sub-class of ruamel.yaml.scalarint.HexCapsInt to store line numbers."""

    def __new__(cls, *args: Any, **kw: Any) -> Any:
        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
        ret_val = super().__new__(cls, *args, **kw)
        ret_val.lc = None
        return ret_val


# class DecimalInt(ruamel.yaml.scalarint.DecimalInt):
#    """Sub-class of ruamel.yaml.scalarint.DecimalInt to store line numbers."""
#
#    def __new__(cls, *args: Any, **kw: Any) -> Any:
#        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
#        ret_val = super().__new__(cls, *args, **kw)
#        ret_val.lc = None
#        return ret_val


class ScalarBoolean(ruamel.yaml.scalarbool.ScalarBoolean):
    """Sub-class of ruamel.yaml.scalarbool.ScalarBoolean to store line numbers."""

    def __new__(cls, *args: Any, **kw: Any) -> Any:
        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
        ret_val = super().__new__(cls, *args, **kw)
        ret_val.lc = None
        return ret_val


class ScalarFloat(ruamel.yaml.scalarfloat.ScalarFloat):
    """Sub-class of to store line numbers."""

    def __new__(cls, *args: Any, **kw: Any) -> Any:
        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
        ret_val = super().__new__(cls, *args, **kw)
        ret_val.lc = None
        return ret_val


# class ExponentialFloat(ruamel.yaml.scalarfloat.ExponentialFloat):
#    """Sub-class of to store line numbers."""
#
#    def __new__(cls, *args: Any, **kw: Any) -> Any:
#        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
#        ret_val = super().__new__(cls, *args, **kw)
#        ret_val.lc = None
#        return ret_val
#
#
# class ExponentialCapsFloat(ruamel.yaml.scalarfloat.ExponentialCapsFloat):
#    """Sub-class of to store line numbers."""
#
#    def __new__(cls, *args: Any, **kw: Any) -> Any:
#        """Constructs a new class instance with the `lc` attribute (`LineCol`)."""
#        ret_val = super().__new__(cls, *args, **kw)
#        ret_val.lc = None
#        return ret_val


StyledScalarString = Union[
    "LiteralScalarString",
    "FoldedScalarString",
    "DoubleQuotedScalarString",
    "SingleQuotedScalarString",
    "PlainScalarString",
    "PreservedScalarString",
]


class RoundTripConstructor(ruamel.yaml.constructor.RoundTripConstructor):
    """
    Round trip constructor that keeps line numbers for most elements.

    Adapted from: https://stackoverflow.com/questions/45716281/parsing-yaml-get-line-numbers-even-in-ordered-maps
    """

    def __init__(self, preserve_quotes: Union[None, bool] = None, loader: Any = None) -> None:
        super().__init__(preserve_quotes=preserve_quotes, loader=loader)
        if not hasattr(self.loader, "comment_handling"):
            self.loader.comment_handling = None

    def _update_lc(self, node: ruamel.yaml.ScalarNode, value: Any) -> None:
        value.lc = ruamel.yaml.comments.LineCol()
        value.lc.line = node.start_mark.line
        value.lc.col = node.start_mark.column

    def _construct_literal_scalar_string(
        self, node: ruamel.yaml.nodes.ScalarNode
    ) -> LiteralScalarString:
        lss = LiteralScalarString(value=node.value, anchor=node.anchor)
        if self.loader and self.loader.comment_handling is None:
            if node.comment and node.comment[1]:
                lss.comment = node.comment[1][0]  # type: ignore[attr-defined]
        else:
            # NEWCMNT
            if node.comment is not None and node.comment[1]:
                # nprintf('>>>>nc1', node.comment)
                # EOL comment after |
                lss.comment = self.comment(node.comment[1][0])  # type: ignore[attr-defined]
        return lss

    def _construct_folded_scalar_string(
        self, node: ruamel.yaml.nodes.ScalarNode
    ) -> FoldedScalarString:
        fold_positions: list[int] = []
        idx = -1
        while True:
            idx = node.value.find("\a", idx + 1)
            if idx < 0:
                break
            fold_positions.append(idx - len(fold_positions))
        fss = FoldedScalarString(value=node.value.replace("\a", ""), anchor=node.anchor)
        if self.loader and self.loader.comment_handling is None:
            if node.comment and node.comment[1]:
                fss.comment = node.comment[1][0]  # type: ignore[attr-defined]
        else:
            # NEWCMNT
            if node.comment is not None and node.comment[1]:
                # nprintf('>>>>nc2', node.comment)
                # EOL comment after >
                fss.comment = self.comment(node.comment[1][0])  # type: ignore[attr-defined]
        if fold_positions:
            fss.fold_pos = fold_positions  # type: ignore[attr-defined]
        return fss

    def construct_scalar(
        self, node: ruamel.yaml.nodes.ScalarNode
    ) -> ruamel.yaml.scalarstring.ScalarString:
        ret_val: Optional[StyledScalarString] = None
        if node.style == "|" and isinstance(node.value, str):
            ret_val = self._construct_literal_scalar_string(node)
        elif node.style == ">" and isinstance(node.value, str):
            ret_val = self._construct_folded_scalar_string(node)
        elif bool(self._preserve_quotes) and isinstance(node.value, str):
            if node.style == "'":
                ret_val = SingleQuotedScalarString(value=node.value, anchor=node.anchor)
            if node.style == '"':
                ret_val = DoubleQuotedScalarString(value=node.value, anchor=node.anchor)
        if ret_val is None:
            if node.anchor is not None:
                ret_val = PlainScalarString(value=node.value, anchor=node.anchor)
            else:
                ret_val = PlainScalarString(value=node.value)
        self._update_lc(node, ret_val)
        return ret_val

    _INT_CLASS_MAP: ClassVar = {
        ruamel.yaml.scalarint.BinaryInt: BinaryInt,
        ruamel.yaml.scalarint.OctalInt: OctalInt,
        ruamel.yaml.scalarint.HexInt: HexInt,
        ruamel.yaml.scalarint.HexCapsInt: HexCapsInt,
        # ruamel.yaml.scalarint.DecimalInt: DecimalInt,
        # this needs to be last as the other classes above are sub-classes
        ruamel.yaml.scalarint.ScalarInt: ScalarInt,
    }

    def construct_yaml_int(self, node: ruamel.yaml.ScalarNode) -> Any:
        super_value = super().construct_yaml_int(node)
        ret_val: Optional[Union[int, ruamel.yaml.scalarint.ScalarInt]] = None
        for src_clazz, dst_clazz in self._INT_CLASS_MAP.items():
            if isinstance(super_value, src_clazz):
                ret_val = dst_clazz(super_value, anchor=node.anchor)
                break
        else:
            ret_val = ScalarInt(super_value, anchor=node.anchor)
        if isinstance(super_value, ruamel.yaml.scalarint.ScalarInt):
            ret_val.__dict__.update(super_value.__dict__)
        self._update_lc(node, ret_val)
        assert ret_val is not None  # for the type checker
        return ret_val

    _FLOAT_CLASS_MAP: ClassVar = {
        # ruamel.yaml.scalarfloat.ExponentialFloat: ExponentialFloat,
        # ruamel.yaml.scalarfloat.ExponentialCapsFloat: ExponentialCapsFloat,
        # this needs to be last as the other classes above are sub-classes
        ruamel.yaml.scalarfloat.ScalarFloat: ScalarFloat,
    }

    def construct_yaml_float(self, node: ruamel.yaml.ScalarNode) -> Union[float, ScalarFloat]:
        super_value: Union[float, ruamel.yaml.scalarfloat.ScalarFloat] = (
            super().construct_yaml_float(node)
        )
        ret_val: Optional[Union[float, ruamel.yaml.scalarfloat.ScalarFloat]] = None
        for src_clazz, dst_clazz in self._FLOAT_CLASS_MAP.items():
            if isinstance(super_value, src_clazz):
                ret_val = dst_clazz(super_value, anchor=node.anchor)
                break
        else:
            ret_val = ScalarFloat(super_value, anchor=node.anchor)
        if isinstance(super_value, ruamel.yaml.scalarfloat.ScalarFloat):
            ret_val.__dict__.update(super_value.__dict__)
        assert ret_val is not None  # for the type checker
        self._update_lc(node, ret_val)
        return ret_val

    def construct_yaml_bool(self, node: ruamel.yaml.ScalarNode) -> ScalarBoolean:  # type: ignore[explicit-override, override]
        super_value: Union[bool, ruamel.yaml.scalarbool.ScalarBoolean] = (
            super().construct_yaml_bool(node)
        )
        ret_val: ScalarBoolean = ScalarBoolean(super_value, anchor=node.anchor)
        if isinstance(super_value, ruamel.yaml.scalarbool.ScalarBoolean):
            ret_val.__dict__.update(super_value.__dict__)
        self._update_lc(node, ret_val)
        return ret_val
